package com.persagy.apm.diagnose.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.framework.ems.data.pojo.hbase.MonitorIndicatorRecord;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;


/**
 * @author leifeng
 * @version 1.0.0
 * @ClassName CollectDataUtil
 * @Description 报警服务发送数据工具类
 * @createTime 2021/9/25 18:41
 */
public class CollectDataUtil {

    public static String buildSendParam(List<MonitorIndicatorRecord> recordList, String alarmItemCode) throws Exception {

        if (CollectionUtils.isEmpty(recordList)) {
            return null;
        }
        StringBuilder stringBuilder = new StringBuilder();
        MonitorIndicatorRecord record1 = recordList.get(0);
        String date = DateUtils.date2Str(record1.getDataTime(), "yyyyMMddHHmmss");
        stringBuilder.append(date).append(";");
        stringBuilder.append(alarmItemCode).append(";");
        stringBuilder.append(record1.getObjId()).append(";");

        for (int i = 0; i < recordList.size(); i++) {
            stringBuilder.append(recordList.get(i).getMonitorIndicatorId());
            if (i == (recordList.size() - 1)) {
                stringBuilder.append(";");
            } else {
                stringBuilder.append(",");
            }
        }
        for (int i = 0; i < recordList.size(); i++) {
            stringBuilder.append(recordList.get(i).getDataValue());
            if (i == (recordList.size() - 1)) {
                stringBuilder.append(";");
            } else {
                stringBuilder.append(",");
            }
        }
        return stringBuilder.toString();

    }


    public static String batchBuildSendParam(Map<String, List<MonitorIndicatorRecord>> sendTimeKeyAndDataList, String alarmItemCode) {
        StringBuilder stringBuilder = new StringBuilder();

        for (Map.Entry<String, List<MonitorIndicatorRecord>> entry : sendTimeKeyAndDataList.entrySet()) {
            String date = entry.getKey();
            stringBuilder.append(date).append(";");
            List<MonitorIndicatorRecord> list = entry.getValue();
            MonitorIndicatorRecord record1 = list.get(0);
            stringBuilder.append(alarmItemCode).append(";");
            stringBuilder.append(record1.getObjId()).append(";");
            for (int i = 0; i < list.size(); i++) {
                stringBuilder.append(list.get(i).getMonitorIndicatorId());
                if (i == (list.size() - 1)) {
                    stringBuilder.append(";");
                } else {
                    stringBuilder.append(",");
                }
            }
            for (int i = 0; i < list.size(); i++) {
                stringBuilder.append(list.get(i).getDataValue());
                if (i == (list.size() - 1)) {
                    stringBuilder.append(";");
                } else {
                    stringBuilder.append(",");
                }
            }
        }
        return stringBuilder.toString();

    }

    public static JSONArray batchBuildSendJsonParam(Map<String, List<MonitorIndicatorRecord>> sendTimeKeyAndDataList, String alarmItemCode) {
        JSONArray array = new JSONArray();
        for (Map.Entry<String, List<MonitorIndicatorRecord>> entry : sendTimeKeyAndDataList.entrySet()) {
            String date = entry.getKey();
            List<MonitorIndicatorRecord> list = entry.getValue();
            if (!CollectionUtils.isEmpty(list) && list.size() > 0){
                for (int i = 0; i < list.size(); i++) {
                    MonitorIndicatorRecord record = list.get(i);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("dataTime",date);
                    jsonObject.put("itemCode",alarmItemCode);
                    jsonObject.put("meterId",record.getObjId());
                    jsonObject.put("funcId",record.getMonitorIndicatorId());
                    jsonObject.put("value",record.getDataValue());
                    array.add(jsonObject);
                }
            }
        }
//        return JSON.toJSONString(array);
        return array;
    }
}
